<?php

/**
 * @file
 * Contains \Drupal\ultimenu\Plugin\Block\UltimenuBlock.
 */

namespace Drupal\ultimenu\Plugin\Block;

use Drupal\block\BlockBase;
use Drupal\block\Annotation\Block;
use Drupal\Core\Annotation\Translation;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides an 'Ultimenu' block.
 *
 * @Block(
 *   id = "ultimenu_block",
 *   admin_label = @Translation("Ultimenu"),
 *   category = @Translation("Ultimenu"),
 *   derivative = "Drupal\ultimenu\Plugin\Derivative\UltimenuBlock"
 * )
 */
class UltimenuBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return array(
      'skin' => 'module|ultimenu--basic',
      'orientation' => 'ultimenu--htb',
    );
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account) {
    return $account->hasPermission('access content');
  }  

  /**
   * Overrides \Drupal\block\BlockBase::blockForm().
   */
  public function blockForm($form, &$form_state) {
    // @see https://drupal.org/node/2049309
    $account = \Drupal::currentUser();

    list($plugin, $derivative) = explode(':', $this->getPluginId());
    // Derivatives are prefixed with 'ultimenu-'.
    $delta = substr($derivative, 9);

    if ($account->hasPermission('administer ultimenu')) {
      $safe_delta = ultimenu_truncate_menu_property($delta);
      $orientation = $this->configuration['orientation'];

      $form['skin'] = array(
        '#type' => 'select',
        '#title' => t('Ultimenu skin'),
        '#default_value' => $this->configuration['skin'],
        '#options' => ultimenu_get_skins(),
        '#empty_option' => t('- None -'),
        '#description' => t('Choose the skin for this block. You can supply custom skins at <a href="@skin_path">Ultimenu settings</a>. The skin can be made specific to this block using the proper class by each menu name.', array('@skin_path' => url('admin/structure/ultimenu'))),
      );

      $form['orientation'] = array(
        '#type' => 'select',
        '#title' => t('Flyout orientation'),
        '#default_value' => !empty($orientation) ? $orientation : 'ultimenu--basic',
        '#options' => array(
          'ultimenu--htb' => t('Horizontal to bottom'),
          'ultimenu--htt' => t('Horizontal to top'),
          'ultimenu--vtl' => t('Vertical to left'),
          'ultimenu--vtr' => t('Vertical to right'),
        ),
        '#description' => t('Choose the orientation of the flyout, depending on the placement. At sidebar left, <strong>Vertical to right</strong>. At header, <strong>Horizontal to bottom</strong>. At footer, <strong>Horizontal to top</strong>'),
      );
    }
    return $form;
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockSubmit().
   */
  public function blockSubmit($form, &$form_state) {
    $this->configuration['skin'] = $form_state['values']['skin'];
    $this->configuration['orientation'] = $form_state['values']['orientation'];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    list($plugin, $derivative) = explode(':', $this->getPluginId());
    // Derivatives are prefixed with 'ultimenu-'.
    $delta = substr($derivative, 9);

    // If no menu link was found, don't display the block.
    if (empty($delta)) {
      return array();
    }

    $skin          = $this->configuration['skin'];
    $orientation   = $this->configuration['orientation'];
    $provider      = '';
    $skin_name     = '';
    $skin_css_path = '';

    // Load specific block skin.
    if (!empty($skin)) {
      $skin_css_path = ultimenu_get_skins_uri($skin);
      // Fetch the skin file name from the setting.
      list($provider, $skin_name) = array_pad(array_map('trim', explode("|", $skin, 2)), 2, NULL);
    }

    // Get the full, un-pruned tree.
    // @todo use menu_navigation_links() since we don't support sub-menus,
    // or use menu_build_tree() for future exploration.
    $tree = menu_tree_all_data($delta);

    // Allow alteration of the tree before we begin operations on it.
    drupal_alter('ultimenu_tree', $tree);

    // Localize the tree.
    // if (module_exists('i18n_menu')) {
      // $tree = i18n_menu_localize_tree($tree);
    // }

    // Provide the settings for ultimenu_tree_output().
    $configs = array(
      'skin'          => $skin,
      'orientation'   => $orientation,
      'delta'         => $delta,
      'menu_name'     => $delta,
      'skin_name'     => $skin_name,
      'skin_provider' => $provider,
    );

    // Build the block.
    $data = array();
    if (($items = ultimenu_tree_output($tree, $configs)) !== NULL) {
      $data = array(
        '#theme'   => array(
          'ultimenu__' . str_replace('-', '_', $delta),
          'ultimenu',
        ),
        '#items'   => $items,
        '#config'  => $configs,
        '#delta'   => $delta,
        '#attached' => array(
          'library' => array(
            array('ultimenu', 'drupal.ultimenu')
          ),
          'css' => array($skin_css_path),
        ),
      );
    }
    return $data;
  }
}
